        // A driver program to test all of the state transitions of
        // an implementation of the StopWatch interface.
        // This main method tests the StrictTimingDevice implementation,
        // which makes use of the ERROR state.
class StrictTimerMain {
    // Define values to represent each of the interface's
    // state transition methods.
    public static final int RESET_METHOD = 0,
                            START_METHOD = 1,
                            STOP_METHOD = 2,
                            SUSPEND_METHOD = 3,
                            RESUME_METHOD = 4;
    public static final int numTransitions = 5;

    public static void main(String[] args){
        // The implementation to be tested.
        final StopWatch timer = new StrictTimer();
        // The set of valid states.
        final int[] validStates = {
            StopWatch.RESET, StopWatch.RUNNING,
            StopWatch.SUSPENDED, StopWatch.STOPPED,
        };
        final int numValidStates = validStates.length;

        // Check that each valid state can be reached.
        for(int i = 0; i < numValidStates; i++){
            gotoState(i,timer);
        }

        System.out.println("All states reachable.");

        // Assume everything will be ok with the state transitions.
        boolean ok = true;
        final int[][] transitions = buildValidTransitions(validStates);
        // Check all transitions from each state.
        for(int state = 0; state < numValidStates; state++){
            for(int transition = 0; transition < numTransitions;
                                        transition++){
                // Put the timer in the correct state for the
                // transition.
                gotoState(state,timer);
                // Make the transition.
                invokeMethod(transition,timer);
                // Check the resulting state.
                if(timer.getState() != transitions[state][transition]){
                    System.err.println("Transition from: "+
                                       state+" on "+
                                       transition+" failed.");
                    System.err.println(timer.getState()+
                                       " reached, instead of "+
                                       transitions[state][transition]);
                    ok = false;
                }
            }
        }
        if(ok){
            System.out.println("All tests passed.");
        }
    }

        // Build a table of all valid state transitions.
        // All non-valid transitions are filled with ERROR.
    public static int[][] buildValidTransitions(int[] validStates){
        final int numValidStates = validStates.length;
        int[][] transitions = new int[numValidStates+1]
                                     [numValidStates+1];
        for(int from = 0; from < numValidStates; from++){
            for(int transition = 0; transition < numTransitions;
                                                transition++){
                transitions[from][transition] = StopWatch.ERROR;
            }
        }

        // Fill in the valid transitions.
        transitions[StopWatch.RESET][RESET_METHOD] = StopWatch.RESET;
        transitions[StopWatch.RESET][START_METHOD] = StopWatch.RUNNING;

        transitions[StopWatch.RUNNING][RESET_METHOD] = StopWatch.RESET;
        transitions[StopWatch.RUNNING][SUSPEND_METHOD] = StopWatch.SUSPENDED;
        transitions[StopWatch.RUNNING][STOP_METHOD] = StopWatch.STOPPED;

        transitions[StopWatch.SUSPENDED][SUSPEND_METHOD] = StopWatch.SUSPENDED;
        transitions[StopWatch.SUSPENDED][RESET_METHOD] = StopWatch.RESET;
        transitions[StopWatch.SUSPENDED][RESUME_METHOD] = StopWatch.RUNNING;
        transitions[StopWatch.SUSPENDED][STOP_METHOD] = StopWatch.STOPPED;

        transitions[StopWatch.STOPPED][STOP_METHOD] = StopWatch.STOPPED;
        transitions[StopWatch.STOPPED][RESET_METHOD] = StopWatch.RESET;

        return transitions;
    }

        // Attempt to place the watch in the given state.
        // The watch is always reset before selecting the desired
        // state.
    public static void gotoState(int state,StopWatch watch)
                                    throws RuntimeException {
        watch.reset();
        switch(state){
            case StopWatch.RESET:
                break;
            case StopWatch.RUNNING:
                watch.start();
                break;
            case StopWatch.SUSPENDED:
                watch.start();
                watch.suspend();
                break;
            case StopWatch.STOPPED:
                watch.start();
                watch.stop();
                break;
            default:
                throw new RuntimeException("Cannot goto state: "+state);
        }
        if(watch.getState() != state){
                throw new RuntimeException("Failed to goto state: "+state);
        }
    }

        // Invoke the given method on watch.
    public static void invokeMethod(int method,StopWatch watch)
                throws RuntimeException {
        switch(method){
            case RESET_METHOD:
                watch.reset();
                break;
            case START_METHOD:
                watch.start();
                break;
            case STOP_METHOD:
                watch.stop();
                break;
            case SUSPEND_METHOD:
                watch.suspend();
                break;
            case RESUME_METHOD:
                watch.resume();
                break;
            default:
                throw new RuntimeException("Cannot select method: "+
                                           method);
        }
    }
}
